/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:27:49.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.core.print.common.jpa.OrderBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.jpa.SqlWrapper;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.CourseStatistic;
import cn.com.qmth.examcloud.core.print.enums.ExamType;
import cn.com.qmth.examcloud.core.print.enums.PaperBindingStatus;
import cn.com.qmth.examcloud.core.print.repository.CourseStatisticRepository;
import cn.com.qmth.examcloud.core.print.service.CommonService;
import cn.com.qmth.examcloud.core.print.service.CourseStatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamCourseInfo;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamInfo;
import cn.com.qmth.examcloud.core.print.service.bean.coursestatistic.CourseStatisticConvert;
import cn.com.qmth.examcloud.core.print.service.bean.coursestatistic.CourseStatisticInfo;
import cn.com.qmth.examcloud.core.print.service.bean.coursestatistic.CourseStatisticQuery;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class CourseStatisticServiceImpl implements CourseStatisticService {
    private static final Logger log = LoggerFactory.getLogger(CourseStatisticServiceImpl.class);
    @Autowired
    private CourseStatisticRepository courseStatisticsRepository;
    @Autowired
    private CommonService commonService;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<CourseStatisticInfo> getCourseStatisticList(CourseStatisticQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        //查询条件
        SearchBuilder searches = new SearchBuilder();
        if (query.getOrgId() != null) {
            searches.eq("orgId", query.getOrgId());
        }
        if (query.getExamId() != null) {
            searches.eq("examId", query.getExamId());
        }
        if (query.getCourseId() != null) {
            searches.eq("courseId", query.getCourseId());
        }
        if (query.getPaperBindingStatus() != null) {
            searches.eq("paperBindingStatus", query.getPaperBindingStatus());
        }
        if (StringUtils.isBlank(query.getPaperName())) {
            searches.like("coursePaper.paperName", query.getPaperName());
        }
        Specification<CourseStatistic> spec = SpecUtils.buildSearchers(CourseStatistic.class, searches.build());
        //排序条件
        OrderBuilder orders = new OrderBuilder().desc("id");
        //分页条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize(), orders.build());
        Page<CourseStatistic> page = courseStatisticsRepository.findAll(spec, pageable);
        return CourseStatisticConvert.ofPage(page);
    }

    @Override
    public void initCourseStatistic(Long orgId, Long examId, Long[] courseIds) {
        List<ExamCourseInfo> examCourses = commonService.getExamCourseList(orgId, examId);
        //courseIds paperType todo
        this.syncCourseStatisticList(examCourses);
    }

    @Override
    public void initAllCourseStatistic() {
        log.debug("initAllCourseStatistic...");
        List<ExamInfo> exams = commonService.getExamList(ExamType.TRADITION.name());
        if (exams != null && !exams.isEmpty()) {
            for (ExamInfo info : exams) {
                List<ExamCourseInfo> examCourses = commonService.getExamCourseList(info.getOrgId(), info.getExamId());
                this.syncCourseStatisticList(examCourses);
            }
        }
    }

    private void syncCourseStatisticList(List<ExamCourseInfo> examCourses) {
        if (examCourses == null || examCourses.isEmpty()) {
            return;
        }
        for (ExamCourseInfo info : examCourses) {
            if (info.getCourseId() == null) {
                continue;
            }
            boolean isExist = this.isExistCourseStatistic(info);
            if (isExist) {
                this.syncCourseStatisticForTotalStudent(info);
            } else {
                this.addCourseStatistic(info);
            }
        }
    }

    private boolean isExistCourseStatistic(ExamCourseInfo info) {
        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", info.getOrgId())
                .eq("examId", info.getExamId())
                .eq("courseId", info.getCourseId())
                .eq("paperType", info.getPaperType());
        Specification<CourseStatistic> spec = SpecUtils.buildSearchers(CourseStatistic.class, searches.build());
        long total = courseStatisticsRepository.count(spec);
        if (total > 0) {
            return true;
        }
        return false;
    }

    private void addCourseStatistic(ExamCourseInfo info) {
        CourseStatistic statistic = new CourseStatistic();
        statistic.setOrgId(info.getOrgId());
        statistic.setExamId(info.getExamId());
        statistic.setCourseId(info.getCourseId());
        statistic.setCourseCode(info.getCourseCode());
        statistic.setCourseName(info.getCourseName());
        statistic.setPaperType(info.getPaperType());
        statistic.setTotalStudent(info.getTotalStudent());
        statistic.setPaperBindingStatus(PaperBindingStatus.无.getIndex());
        courseStatisticsRepository.save(statistic);
    }

    private void syncCourseStatisticForTotalStudent(ExamCourseInfo info) {
        //仅更新total_student字段
        SqlWrapper sql = new SqlWrapper()
                .update("ec_prt_course_statistic")
                .set().append("total_student = ").append(info.getTotalStudent())
                .where()
                .eq("org_id", info.getOrgId())
                .and().eq("exam_id", info.getExamId())
                .and().eq("course_id", info.getCourseId())
                .and().eq("paper_type", info.getPaperType());
        jdbcTemplate.update(sql.build());
    }

    @Override
    public void syncCourseStatisticForCourseName(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        SqlWrapper sql = new SqlWrapper()
                .update("ec_prt_course_statistic")
                .set().append("course_name = '").append(courseName).append("'")
                .where()
                .eq("course_id", courseId);
        jdbcTemplate.update(sql.build());
    }

}